% Check if first order system
function tunefopdt1(action);

   
[g,h]=gcbo;
hK=findobj(h,'Tag','K');
htau=findobj(h,'Tag','Tau');
htheta=findobj(h,'Tag','Delay');
hrule=findobj(h,'Tag','Rule');
hnoise=findobj(h,'Tag','Noise');
hkc=findobj(h,'Tag','Kc');
htaui=findobj(h,'Tag','Taui');
htaud=findobj(h,'Tag','Taud');
halpha=findobj(h,'Tag','Alpha');
hts=findobj(h,'Tag','Ts');
hgraph=findobj(h,'Tag','myplot');
hcontrol=findobj(h,'Tag','Control');
hcomax=findobj(h,'Tag','CO_Max');
hcomin=findobj(h,'Tag','CO_Min');
hmvmax=findobj(h,'Tag','MV_Max');
hmvmin=findobj(h,'Tag','MV_Min');


K=str2num(get(hK,'String'));
if K< 0 ; K=-K; end;
tau=str2num(get(htau,'String'));
theta=str2num(get(htheta,'String'));
if tau < 0; 
   message ='Time Constant must be positive';
   errordlg(message,'Error in input',replace);
   return;
end;
if theta < 0; 
   message ='Delay must be positive or zero';
   errordlg(message,'Error in input',replace);
   return;
end;

ts=str2num(get(hts,'String'));
rule=get(hrule,'Value');
noise=get(hnoise,'Value');
control=str2num(get(hcontrol,'String'));
comax=str2num(get(hcomax,'String'));
comin=str2num(get(hcomin,'String'));
mvmax=str2num(get(hmvmax,'String'));
mvmin=str2num(get(hmvmin,'String'));
kc=str2num(get(hkc,'String'));
taui=str2num(get(htaui,'String'));
taud=str2num(get(htaud,'String'));
alpha=str2num(get(halpha,'String'));
K=str2num(get(hK,'String'));
tau=str2num(get(htau,'String'));
theta=str2num(get(htheta,'String'));

switch ( action) ;
  case   'design'
   design_fn(K,tau,theta,ts,rule,noise,control,hkc,htaui,htaud,halpha)
case 'set_response'
   setresponse_fn(K,tau,theta,ts,kc,taui,taud,alpha,comax,comin,mvmax,mvmin,hgraph)
case 'load_response'
   loadresponse_fn(K,tau,theta,ts,kc,taui,taud,alpha,comax,comin,mvmax,mvmin,hgraph)   
end


function design_fn(K,tau,theta,ts,rule,noise,control,hkc,htaui,htaud,halpha)
% function to compute controller tuning constants
%


switch rule,
case 1 %IMC Brosilow PID controller
 
   uncertain =(control)./100.0;
   tcl=max([6*uncertain*theta,(0.6*uncertain+.14)*tau]);
   
      Kc=(tau+theta^2/(2*(tcl+theta)))/(K*(tcl+theta));
      taui=tau+theta^2/(2*(tcl+theta));
      taud=theta^2/(2*(tcl+theta))*(1-theta/(3*taui));
      if taud < 3*ts % taud is too small compared to sample time
         taud=0; % use no derivative action
         alpha =0; % no filter
      else
         alpha=max([0.1*taud,3*ts]); % alpha cannot be smaller than 3ts.
      end
        

case 2  % Lopez Setpoint PID
        if (theta <= 0) 
               message={' Cannot use Lopez Rule if delay is zero',...
                     ' Use IMC-Brosilow Rule Instead'};

			errordlg(message,'Error in input','replace');
			end;

      Kc=1.086/K*(theta/tau)^(-0.869);taui=tau/(0.740-.130*(theta/tau));
      taud=0.348*tau*(theta/tau)^.914;alpha=.1*taud;
      
case 3 % Lopez Load PID
         if (theta <= 0) 
               message={' Cannot use Lopez Rule if delay is zero',...
                     ' Use IMC-Brosilow Rule Instead'};

			errordlg(message,'Error in input','replace');
			end;

Kc=1.357/K*(theta/tau)^(-0.947);taui=tau/.842*(theta/tau)^.738;
taud=.381*tau*(theta/tau)^.995;alpha=.1*taud;

case 4 %Ziegler Nichols rule for PID of FOPDT process
   if (theta <= 0) 
      message={' Cannot use ZN Rule if delay is zero',...
                     ' Use IMC-Brosilow Rule Instead'};

      errordlg(message,'Error in input','replace');
      return;
   end;
   
   Kc=1.2*tau/(K*theta);
   taui=2*theta;
   taud=0.5*theta;   
   %alpha=0;
   alpha=max(0.1*taud,3*ts);  
case 5 %Ziegler Nichols for PI
      if (theta <= 0) 
               message={' Cannot use ZN Rule if delay is zero',...
                     ' Use IMC-Brosilow Rule Instead'};

         errordlg(message,'Error in input','replace');
      return;
   end;

   Kc=0.9*tau/(K*theta);
   taui=0.33*theta;
   taud=0;  
   alpha=0;
case 6 %Ziegler Nichols for P
      if (theta <= 0) 
               message={' Cannot use ZN Rule if delay is zero',...
                     ' Use IMC-Brosilow Rule Instead'};
      errordlg(message,'Error in input','replace');
      return;
   end;

   Kc=tau/(K*theta);
   taui=99999;
   taud=0;   
   alpha=0;
case 7 %Cohen-Coon rule for PID
      if (theta <= 0) 
         message={' Cannot use Cohen-Coon Rule if delay is zero',...
            ' Use IMC-Brosilow Rule Instead'};
      errordlg(message,'Error in input','replace');
      return;
   end;

   Kc=(tau/theta)*(4/3+theta/(4*tau))/K;
   taui=theta*(32+6*theta/tau)/(13+8*theta/tau);
   taud=4*theta/(11+2*theta/tau);
   %alpha=0;
   alpha=max(0.1*taud,3*ts);  
case 8 %Coehn Coon for PI
         if (theta <= 0) 
         message={' Cannot use Cohen-Coon Rule if delay is zero',...
            ' Use IMC-Brosilow Rule Instead'};
      errordlg(message,'Error in input','replace');
      return;
   end;

   Kc=(tau/theta)*(0.9+theta/(12*tau))/K;
   taui=theta*(30+3*theta/tau)/(9+20*theta/tau);
   taud=0;
   alpha=0;
case 9 %Cohen Coon for P
         if (theta <= 0) 
         message={' Cannot use Cohen-Coon Rule if delay is zero',...
            ' Use IMC-Brosilow Rule Instead'};
      errordlg(message,'Error in input','replace');
      return;
   end;

   Kc=(tau/theta)*(1+theta/(3*tau))/K;
   taui=9999;
   taud=0;
   alpha=0;
end
if (noise==1)
   taud=0;
end
if (alpha>taud)
   alpha=0;
   taud=0;
end
set(hkc,'String',num2str(Kc));
set(htaui,'String',num2str(taui));
set(htaud,'String',num2str(taud));
set(halpha,'String',num2str(alpha));
return
% end of design function

% Start of set point response calculation
%
   function setresponse_fn(K,tau,theta,ts,kc,taui,taud,alpha,comax,comin,mvmax,mvmin,hgraph)
   %
   %
   % call simulink model to generate response
   % plot the response
   %
   options=simset('SrcWorkspace','current');
   timespan=[0 4*(tau+theta)];
   ut=[ [ 0 4*(tau+theta)]',[1 1]',[0 0]']; 
   [g,h]=gcbo;
   hgraph=findobj(h,'Tag','myplot');

   [t,x,y]=sim('fopdt_pid',timespan,options,ut);

   axes(hgraph);
   plot(t,y(:,1),'b-',t,y(:,2),'r--');grid on;xlabel('Time');
   ylabel('Output(Blue);Manip Var (Red)');
   title('Response to a Unit change in Set Point, IDEAL PID');
   set(hgraph,'Tag','myplot');
   return
   
   function loadresponse_fn(K,tau,theta,ts,kc,taui,taud,alpha,comax,comin,mvmax,mvmin,hgraph) ;
   
   options=simset('SrcWorkspace','current');
   timespan=[0 4*(tau+theta)];

   ut=[ [ 0 4*(tau+theta)]',[0 0]',[1 1]']; 
   [g,h]=gcbo;
   hgraph=findobj(h,'Tag','myplot');

   [t,x,y]=sim('fopdt_pid',timespan,options,ut);

   axes(hgraph);
   plot(t,y(:,1),'b-',t,y(:,2),'r--');grid on;xlabel('Time');
   ylabel('Output(Blue);Manip Var (Red)');
   title({'Response to a Unit change in Load Dist., IDEAL PID',...
         'Process and Disturbance transfer function are assumed equal'},'Color','red');
   set(hgraph,'Tag','myplot');
   return

%get response and plot open loop and closed loop responses
num1=[0 K];
den1=[tau 1];
[num2,den2]=pade(theta,5);
nump=conv(num1,num2);
denp=conv(den1,den2);
sysp=tf(nump,denp);
numc=Kc*([taui*taud*(1+alpha) taui+alpha*taud 1]);
denc=[alpha*taui*taud taui 0];
sysc= tf(numc,denc); %put controller transfer function here
syscl=feedback(series(sysc,sysp),1);
sysco=feedback(sysc,sysp);
[y,t]=step(syscl);
[c,t]=step(sysco,t);
plot(t,y,'b-',t,c,'r--');
grid on;
xlabel('Time');
ylabel('PV(blue)/CO(red)');

